@@ -494,11 +494,8 @@ class GlobalDatabasesMetricGroup(MetricGroupDef):
             # Add system metrics from pg_stat_kcache,
             kcache_query = kcache_getstatdata_sample("db")
 
-            total_sys_hit = (
-                "{total_read} - sum(sub.reads)"
-                "/ {ts}"
-                " AS total_sys_hit"
-                "".format(total_read=total_read("sub", True), ts=get_ts())
+            total_sys_hit = "greatest({total_read} - sum(sub.reads)/ {ts},0) AS total_sys_hit".format(
+                total_read=total_read("sub", True), ts=get_ts()
             )
             total_disk_read = (
                 "sum(sub.reads) / " + get_ts() + " AS total_disk_read"
@@ -640,6 +637,7 @@ class GlobalPGSAMetricGroup(MetricGroupDef):
 
     name = "pgsa"
     xaxis = "ts"
+    desc = "All backends, excluding autovacuum workers, are included"
     data_url = r"/server/(\d+)/metrics/pgsa/"
     backend_xid_age = MetricDef(label="Backend xid age")
     backend_xmin_age = MetricDef(label="Backend xmin age")
@@ -675,14 +673,14 @@ class GlobalPGSAMetricGroup(MetricGroupDef):
     @property
     def query(self):
         query = powa_get_pgsa_sample()
-
+        # Those greatests are to avoid having negative values returned, see https://github.com/powa-team/powa-archivist/issues/98
         cols = [
             "extract(epoch FROM ts) AS ts",
-            "max(backend_xid_age) AS backend_xid_age",
-            "max(backend_xmin_age) AS backend_xmin_age",
-            "max(backend_start_age) FILTER (WHERE datid IS NOT NULL) AS oldest_backend",
-            "max(xact_start_age) FILTER (WHERE datid IS NOT NULL) AS oldest_xact",
-            "max(query_start_age) FILTER (WHERE datid IS NOT NULL) AS oldest_query",
+            "greatest(max(backend_xid_age),0) AS backend_xid_age",
+            "greatest(max(backend_xmin_age),0) AS backend_xmin_age",
+            "greatest(max(backend_start_age) FILTER (WHERE datid IS NOT NULL),0) AS oldest_backend",
+            "greatest(max(xact_start_age) FILTER (WHERE datid IS NOT NULL),0) AS oldest_xact",
+            "greatest(max(query_start_age) FILTER (WHERE datid IS NOT NULL AND state !~ 'idle'),0) AS oldest_query",
             "count(*) FILTER (WHERE state = 'idle') AS nb_idle",
             "count(*) FILTER (WHERE state = 'active') AS nb_active",
             "count(*) FILTER (WHERE state = 'idle in transaction') AS nb_idle_xact",
@@ -1068,8 +1066,7 @@ class GlobalSubMetricGroup(MetricGroupDef):
     apply_error_count = MetricDef(
         label="# apply error",
         type="number",
-        desc="Total number of times an error"
-        " occurred while applying changes",
+        desc="Total number of times an error occurred while applying changes",
     )
     sync_error_count = MetricDef(
         label="# sync error",
@@ -1239,8 +1236,7 @@ class GlobalWalReceiverMetricGroup(MetricGroupDef):
     flush_delta = MetricDef(
         label="Flush delta",
         type="size",
-        desc="Total amount of data received and written"
-        " but not flushed yet",
+        desc="Total amount of data received and written but not flushed yet",
     )
     last_msg_lag = MetricDef(
         label="Last message latency",
@@ -1855,6 +1851,7 @@ class ServerOverview(DashboardPage):
                 Graph(
                     "Global activity (all databases)",
                     metrics=pgsa_metrics[0],
+                    desc=GlobalPGSAMetricGroup.desc,
                     renderer="bar",
                     stack=True,
                 )
@@ -1867,7 +1864,11 @@ class ServerOverview(DashboardPage):
                 )
             )
         all_db_graphs[1].append(
-            Graph("Backend age (all databases)", metrics=pgsa_metrics[1])
+            Graph(
+                "Backend age (all databases)",
+                metrics=pgsa_metrics[1],
+                desc=GlobalPGSAMetricGroup.desc,
+            )
         )
 
         graphs_dash = [Dashboard("General Overview", all_db_graphs)]
