@@ -1,13 +1,11 @@
 """
 Utilities for commonly used SQL constructs.
 """
+
 import re
-from sqlalchemy.sql import (text, select, func, column,
-                            and_, literal_column)
-from sqlalchemy.dialects.postgresql import array, dialect as pgdialect
-from collections import namedtuple, defaultdict
-from powa.json import JSONizable
 import sys
+from collections import defaultdict, namedtuple
+from powa.json import JSONizable
 
 TOTAL_MEASURE_INTERVAL = """
 extract( epoch from
@@ -18,9 +16,9 @@ extract( epoch from
 
 
 def unprepare(sql):
-    if sql.startswith('PREPARE'):
-        sql = re.sub('PREPARE.*AS', '', sql)
-    sql = re.sub('\$\d+', '?', sql)
+    if sql.startswith("PREPARE"):
+        sql = re.sub("PREPARE.*AS", "", sql)
+    sql = re.sub("\$\d+", "?", sql)
     return sql
 
 
@@ -34,7 +32,7 @@ def format_jumbled_query(sql, params):
     return sql
 
 
-RESOLVE_OPNAME = text("""
+RESOLVE_OPNAME = """
 SELECT json_object_agg(oid, value)
     FROM (
 
@@ -66,14 +64,14 @@ SELECT json_object_agg(oid, value)
         LEFT JOIN pg_am ON amop.amopmethod = pg_am.oid AND pg_am.amname != 'hash'
         LEFT JOIN pg_opfamily f ON f.opfmethod = pg_am.oid AND amop.amopfamily = f.oid
         LEFT JOIN pg_opclass c ON c.opcfamily = f.oid
-        WHERE pg_operator.oid in :oid_list
+        WHERE pg_operator.oid in %(oid_list)s
         GROUP BY pg_operator.oid, oprname, pg_am.oid, amname
     ) by_am
     GROUP BY oproid, oprname
     ) detail
-""")
+"""
 
-RESOLVE_ATTNAME = text("""
+RESOLVE_ATTNAME = """
     SELECT json_object_agg(attrelid || '.'|| attnum, value)
     FROM (
     SELECT attrelid, attnum, json_build_object(
@@ -97,22 +95,27 @@ RESOLVE_ATTNAME = text("""
     INNER JOIN pg_namespace n ON n.oid = c.relnamespace
     LEFT JOIN pg_statistic s ON s.starelid = c.oid
                        AND s.staattnum = a.attnum
-    WHERE (attrelid, attnum) IN :att_list
+    WHERE (attrelid, attnum) IN %(att_list)s
     ) detail
-""")
+"""
 
 
 class ResolvedQual(JSONizable):
-
-    def __init__(self, nspname, relname, attname,
-                 opname, amops,
-                 n_distinct=0,
-                 most_common_values=None,
-                 null_frac=None,
-                 example_values=None,
-                 eval_type=None,
-                 relid=None,
-                 attnum=None):
+    def __init__(
+        self,
+        nspname,
+        relname,
+        attname,
+        opname,
+        amops,
+        n_distinct=0,
+        most_common_values=None,
+        null_frac=None,
+        example_values=None,
+        eval_type=None,
+        relid=None,
+        attnum=None,
+    ):
         self.nspname = nspname
         self.relname = relname
         self.attname = attname
@@ -127,35 +130,39 @@ class ResolvedQual(JSONizable):
         self.attnum = attnum
 
     def __str__(self):
-        return "%s.%s %s ?" % (self.relname, self.attname, self.opname)
+        return "{}.{} {} ?".format(self.relname, self.attname, self.opname)
 
     @property
     def distinct_values(self):
         if self.n_distinct == 0:
             return None
         elif self.n_distinct > 0:
-            return "%s" % self.n_distinct
+            return "{}".format(self.n_distinct)
         else:
-            return "%s %%" % (abs(self.n_distinct) * 100)
+            return "{} %".format(abs(self.n_distinct) * 100)
 
     def to_json(self):
         base = super(ResolvedQual, self).to_json()
         base["label"] = str(self)
+        base["distinct_values"] = self.distinct_values
         return base
 
 
 class ComposedQual(JSONizable):
-
-    def __init__(self, nspname=None, relname=None,
-                 avg_filter=None,
-                 filter_ratio=None,
-                 occurences=None,
-                 execution_count=None,
-                 table_liverows=None,
-                 qualid=None,
-                 relid=None,
-                 queries=None,
-                 queryids=None):
+    def __init__(
+        self,
+        nspname=None,
+        relname=None,
+        avg_filter=None,
+        filter_ratio=None,
+        occurences=None,
+        execution_count=None,
+        table_liverows=None,
+        qualid=None,
+        relid=None,
+        queries=None,
+        queryids=None,
+    ):
         super(ComposedQual, self).__init__()
         self.qualid = qualid
         self.relname = relname
@@ -172,8 +179,9 @@ class ComposedQual(JSONizable):
 
     def append(self, element):
         if not isinstance(element, ResolvedQual):
-            raise ValueError(("ComposedQual elements must be instances of ",
-                              "ResolvedQual"))
+            raise ValueError(
+                ("ComposedQual elements must be instances of ", "ResolvedQual")
+            )
         self._quals.append(element)
 
     def __iter__(self):
@@ -184,12 +192,12 @@ class ComposedQual(JSONizable):
 
     @property
     def where_clause(self):
-        return "WHERE %s" % self
+        return "WHERE {}".format(self)
 
     def to_json(self):
         base = super(ComposedQual, self).to_json()
-        base['quals'] = self._quals
-        base['where_clause'] = self.where_clause
+        base["quals"] = self._quals
+        base["where_clause"] = self.where_clause
         return base
 
 
@@ -214,77 +222,101 @@ def resolve_quals(conn, quallist, attribute="quals"):
         if not isinstance(values, list):
             values = [values]
         for v in values:
-            operator_to_look.add(v['opno'])
+            operator_to_look.add(v["opno"])
             attname_to_look.add((v["relid"], v["attnum"]))
     if operator_to_look:
-        operators = conn.execute(
-            RESOLVE_OPNAME,
-            {"oid_list": tuple(operator_to_look)}).scalar()
+        cur = conn.cursor()
+        cur.execute(RESOLVE_OPNAME, {"oid_list": tuple(operator_to_look)})
+        operators = cur.fetchone()[0]
+        cur.close()
     if attname_to_look:
-        attnames = conn.execute(
-            RESOLVE_ATTNAME,
-            {"att_list": tuple(attname_to_look)}).scalar()
+        cur = conn.cursor()
+        cur.execute(RESOLVE_ATTNAME, {"att_list": tuple(attname_to_look)})
+        attnames = cur.fetchone()[0]
     new_qual_list = []
     for row in quallist:
         row = dict(row)
         newqual = ComposedQual(
-            occurences=row['occurences'],
-            execution_count=row['execution_count'],
-            avg_filter=row['avg_filter'],
-            filter_ratio=row['filter_ratio'],
-            qualid=row['qualid'],
-            queries=row.get('queries'),
-            queryids=row.get('queryids')
+            occurences=row["occurences"],
+            execution_count=row["execution_count"],
+            avg_filter=row["avg_filter"],
+            filter_ratio=row["filter_ratio"],
+            qualid=row["qualid"],
+            queries=row.get("queries"),
+            queryids=row.get("queryids"),
         )
         new_qual_list.append(newqual)
-        values = [v for v in row[attribute] if v['relid'] != '0']
+        values = [v for v in row[attribute] if v["relid"] != "0"]
         if not isinstance(values, list):
             values = [values]
         for v in values:
-            attname = attnames["%s.%s" % (v["relid"], v["attnum"])]
+            attname = attnames["{}.{}".format(v["relid"], v["attnum"])]
             if newqual.relname is not None:
-                if newqual.relname != attname['relname']:
-                    raise ValueError("All individual qual parts should be on the "
-                                     "same relation")
+                if newqual.relname != attname["relname"]:
+                    raise ValueError(
+                        "All individual qual parts should be on the "
+                        "same relation"
+                    )
             else:
                 newqual.relname = attname["relname"]
                 newqual.nspname = attname["nspname"]
                 newqual.relid = v["relid"]
                 newqual.table_liverows = attname["table_liverows"]
-            newqual.append(ResolvedQual(
-                nspname=attname['nspname'],
-                relname=attname['relname'],
-                attname=attname['attname'],
-                opname=operators[v["opno"]]["name"],
-                amops=operators[v["opno"]]["amop_names"],
-                n_distinct=attname["n_distinct"],
-                most_common_values=attname["most_common_values"],
-                null_frac=attname["null_frac"],
-                eval_type=v["eval_type"],
-                relid=v["relid"],
-                attnum=v["attnum"]))
+            newqual.append(
+                ResolvedQual(
+                    nspname=attname["nspname"],
+                    relname=attname["relname"],
+                    attname=attname["attname"],
+                    opname=operators[v["opno"]]["name"],
+                    amops=operators[v["opno"]]["amop_names"],
+                    n_distinct=attname["n_distinct"],
+                    most_common_values=attname["most_common_values"],
+                    null_frac=attname["null_frac"],
+                    eval_type=v["eval_type"],
+                    relid=v["relid"],
+                    attnum=v["attnum"],
+                )
+            )
     return new_qual_list
 
 
 Plan = namedtuple(
     "Plan",
-    ("title", "values", "query", "plan", "filter_ratio", "exec_count", "occurences"))
+    (
+        "title",
+        "values",
+        "query",
+        "plan",
+        "filter_ratio",
+        "exec_count",
+        "occurences",
+    ),
+)
 
 
-def qual_constants(srvid, type, tsfrom, tsto, filter_clause,
-                   queries=None, quals=None, top=1):
+def qual_constants(
+    srvid, type, filter_clause, queries=None, quals=None, top=1
+):
+    """
+    filter_clause is a plain string corresponding to the list of predicates to
+    apply in the main query.
+
+    queries and quals are optional list of respectively queryid and qualid to
+    filter on.
+    """
     orders = {
-        'most_executed': "8 DESC",
-        'least_filtering': "9",
-        'most_filtering': "9 DESC",
-        'most_used': '6 DESC'
+        "most_executed": "8 DESC",
+        "least_filtering": "9",
+        "most_filtering": "9 DESC",
+        "most_used": "6 DESC",
     }
-    if type not in ('most_executed', 'most_filtering',
-                    'least_filtering', 'most_used'):
+    if type not in (
+        "most_executed",
+        "most_filtering",
+        "least_filtering",
+        "most_used",
+    ):
         return
-    dialect = pgdialect()
-    dialect.paramstyle = 'named'
-    filter_clause = filter_clause.compile(dialect=dialect)
 
     query_subfilter = ""
     query_filter = ""
@@ -292,14 +324,14 @@ def qual_constants(srvid, type, tsfrom, tsto, filter_clause,
     qual_filter = ""
 
     if queries is not None:
-        query_subfilter = "AND queryid IN (%s)" % queries
-        query_filter = "AND s.queryid IN (%s)" % queries
+        query_subfilter = "AND queryid IN ({})".format(queries)
+        query_filter = "AND s.queryid IN ({})".format(queries)
 
     if quals is not None:
-        qual_subfilter = "AND qualid IN (%s)" % quals
-        qual_filter = "AND qnc.qualid IN (%s)" % quals
+        qual_subfilter = "AND qualid IN ({})".format(quals)
+        qual_filter = "AND qnc.qualid IN ({})".format(quals)
 
-    base = text("""
+    base = """
     (
     WITH sample AS (
     SELECT s.srvid, query, s.queryid, qn.qualid, quals as quals,
@@ -308,80 +340,101 @@ def qual_constants(srvid, type, tsfrom, tsto, filter_clause,
                 sum(execution_count) as execution_count,
                 sum(nbfiltered) as nbfiltered,
                 CASE WHEN sum(execution_count) = 0 THEN 0 ELSE sum(nbfiltered) / sum(execution_count) END AS filter_ratio
-        FROM powa_statements s
-        JOIN powa_databases d ON d.oid = s.dbid AND d.srvid = s.srvid
-        JOIN powa_qualstats_quals qn ON s.queryid = qn.queryid AND s.srvid = qn.srvid
+        FROM {{powa}}.powa_statements s
+        JOIN {{powa}}.powa_databases d ON d.oid = s.dbid AND d.srvid = s.srvid
+        JOIN {{powa}}.powa_qualstats_quals qn ON s.queryid = qn.queryid AND s.srvid = qn.srvid
         JOIN (
             SELECT *
-            FROM powa_qualstats_constvalues_history qnc
-            WHERE srvid = :server
-            %(query_subfilter)s
-            %(qual_subfilter)s
-              AND coalesce_range && tstzrange(:from, :to)
+            FROM {{powa}}.powa_qualstats_constvalues_history qnc
+            WHERE srvid = {srvid}
+            {query_subfilter}
+            {qual_subfilter}
+              AND coalesce_range && tstzrange(%(from)s, %(to)s)
             UNION ALL
             SELECT *
-            FROM powa_qualstats_aggregate_constvalues_current(:server, :from, :to)
-            WHERE srvid = :server
-            %(query_subfilter)s
-            %(qual_subfilter)s
+            FROM {{powa}}.powa_qualstats_aggregate_constvalues_current(%(server)s, %(from)s, %(to)s)
+            WHERE srvid = {srvid}
+            {query_subfilter}
+            {qual_subfilter}
         ) qnc ON qnc.srvid = s.srvid AND qn.qualid = qnc.qualid AND qn.queryid = qnc.queryid,
         LATERAL
-                unnest(%(qual_type)s) as t(constants,occurences, execution_count, nbfiltered)
-        WHERE %(filter)s
-        %(query_filter)s
-        %(qual_filter)s
-        AND s.srvid = :server
+                unnest({qual_type}) as t(constants,occurences, execution_count, nbfiltered)
+        WHERE {filter}
+        {query_filter}
+        {qual_filter}
+        AND s.srvid = {srvid}
         GROUP BY s.srvid, qn.qualid, quals, constants, s.queryid, query
-        ORDER BY %(order)s
-        LIMIT :top_value
+        ORDER BY {order}
+        LIMIT {top_value}
     )
     SELECT srvid, query, queryid, qualid, quals, constants as constants,
                 occurences as occurences,
                 nbfiltered as nbfiltered,
                 execution_count as execution_count,
                 filter_ratio as filter_ratio,
-                row_number() OVER (ORDER BY execution_count desc NULLS LAST) as rownumber
+                row_number() OVER (ORDER BY execution_count desc NULLS LAST) AS rownumber
         FROM sample
-    ORDER BY 11
-    LIMIT :top_value
-    ) %(qual_type)s
-    """ % {
-        'qual_type': type,
-        'filter': str(filter_clause),
-        'query_subfilter': str(query_subfilter),
-        'query_filter': str(query_filter),
-        'qual_subfilter': str(qual_subfilter),
-        'qual_filter': str(qual_filter),
-        'order': orders[type]})
-    base = base.params(top_value=top, **filter_clause.params)
-    return select(["*"]).select_from(base)
+    ORDER BY rownumber
+    LIMIT {top_value}
+    ) {qual_type}
+    """.format(
+        qual_type=type,
+        filter=filter_clause,
+        query_subfilter=query_subfilter,
+        query_filter=query_filter,
+        qual_subfilter=qual_subfilter,
+        qual_filter=qual_filter,
+        order=orders[type],
+        top_value=top,
+        srvid=srvid,
+    )
+
+    query = "SELECT * FROM " + base
+
+    return query
 
 
 def quote_ident(name):
     return '"' + name + '"'
 
 
-def get_plans(self, query, database, qual):
+def get_plans(cls, server, database, query, all_vals):
     plans = []
-    for key in ('most filtering', 'least filtering', 'most executed'):
-        vals = qual[key]
-        query = format_jumbled_query(query, vals['constants'])
+    for key in (
+        "most filtering",
+        "least filtering",
+        "most executed",
+        "most used",
+    ):
+        vals = all_vals[key]
+        query = format_jumbled_query(query, vals["constants"])
         plan = "N/A"
         try:
-            result = self.execute("EXPLAIN %s" % query,
-                                  database=database,
-                                  remote_access=True)
-            plan = "\n".join(v[0] for v in result)
-        except Exception:
+            sqlQuery = "EXPLAIN {}".format(query)
+            result = cls.execute(
+                sqlQuery, srvid=server, database=database, remote_access=True
+            )
+            plan = "\n".join(v["QUERY PLAN"] for v in result)
+        except Exception as e:
+            plan = "ERROR: %r" % e
             pass
-        plans.append(Plan(key, vals['constants'], query,
-                          plan, vals["filter_ratio"], vals['execution_count'],
-                          vals['occurences']))
+        plans.append(
+            Plan(
+                key,
+                vals["constants"],
+                query,
+                plan,
+                vals["filter_ratio"],
+                vals["execution_count"],
+                vals["occurences"],
+            )
+        )
     return plans
 
 
-def get_unjumbled_query(ctrl, srvid, database, queryid, _from, _to,
-                        kind='most executed'):
+def get_unjumbled_query(
+    ctrl, srvid, database, queryid, _from, _to, kind="most executed"
+):
     """
     From a queryid, build a query string ready to be executed.
 
@@ -393,15 +446,21 @@ def get_unjumbled_query(ctrl, srvid, database, queryid, _from, _to,
     been found and/or the SELECT clause has been normalized
     """
 
-    rs = list(ctrl.execute(text("""
+    rs = list(
+        ctrl.execute(
+            """
         SELECT query
-        FROM powa_statements
-        WHERE srvid= :srvid
-        AND queryid = :queryid LIMIT 1
-    """), params={"srvid": srvid, "queryid": queryid}))[0]
-    normalized_query = rs[0]
-    values = qualstat_get_figures(ctrl, srvid, database, _from, _to,
-                                  queries=[queryid])
+        FROM {powa}.powa_statements
+        WHERE srvid= %(srvid)s
+        AND queryid = %(queryid)s LIMIT 1
+    """,
+            params={"srvid": srvid, "queryid": queryid},
+        )
+    )[0]
+    normalized_query = rs["query"]
+    values = qualstat_get_figures(
+        ctrl, srvid, database, _from, _to, queries=[queryid]
+    )
 
     if values is None:
         unprepared = unprepare(normalized_query)
@@ -409,8 +468,9 @@ def get_unjumbled_query(ctrl, srvid, database, queryid, _from, _to,
             return None
 
     # Try to inject values
-    sql = format_jumbled_query(normalized_query,
-                               values[kind].get('constants', []))
+    sql = format_jumbled_query(
+        normalized_query, values[kind].get("constants", [])
+    )
     return sql
 
 
@@ -419,7 +479,8 @@ def get_any_sample_query(ctrl, srvid, database, queryid, _from, _to):
     From a queryid, get a non normalized query.
 
     If pg_qualstats is available and recent enough, try to retrieve a randomly
-    chosen non normalized query, which is fast.
+    chosen non normalized query, which is fast, trying to avoid EXPLAIN
+    queries.
 
     If this fail, fallback get_unjumbled_query, with "most executed" const
     values.
@@ -427,79 +488,96 @@ def get_any_sample_query(ctrl, srvid, database, queryid, _from, _to):
     has_pgqs = ctrl.has_extension_version(srvid, "pg_qualstats", "0.0.7")
     example_query = None
     if has_pgqs:
-        rs = list(ctrl.execute(text("""
-            SELECT pg_qualstats_example_query(:queryid)
+        rows = ctrl.execute(
+            """
+            WITH s(max, v) AS (
+                SELECT
+                (SELECT setting::int
+                    FROM pg_catalog.pg_settings
+                    WHERE name = 'track_activity_query_size'
+                ), {pg_qualstats}.pg_qualstats_example_query(%(queryid)s)
+            )
+            SELECT max, v, octet_length(v) AS len
+            FROM s
+            WHERE v NOT ILIKE '%%EXPLAIN%%'
             LIMIT 1
-        """), params={"queryid": queryid}, srvid=srvid, remote_access=True))
-        if len(rs) > 0:
-            example_query = rs[0][0]
+        """,
+            params={"queryid": queryid},
+            srvid=srvid,
+            remote_access=True,
+        )
+        if rows is not None and len(rows) > 0:
+            # Ignore the query if it looks like it was truncated
+            if rows[0]["len"] < (rows[0]["max"] - 1):
+                example_query = rows[0]["v"]
         if example_query is not None:
             unprepared = unprepare(example_query)
             if example_query == unprepared:
                 return example_query
-    return get_unjumbled_query(ctrl, srvid, database, queryid,
-                               _from, _to, 'most executed')
+    return get_unjumbled_query(
+        ctrl, srvid, database, queryid, _from, _to, "most executed"
+    )
 
 
-def qualstat_get_figures(conn, srvid, database, tsfrom, tsto,
-                         queries=None, quals=None):
-    condition = text("""datname = :database
-            AND coalesce_range && tstzrange(:from, :to)""")
+def qualstat_get_figures(
+    cls, srvid, database, tsfrom, tsto, queries=None, quals=None
+):
+    condition = """datname = %(database)s
+            AND coalesce_range && tstzrange(%(from)s, %(to)s)"""
 
     if queries is not None:
-        queries_str = ','.join(str(q) for q in queries)
+        queries_str = ",".join(str(q) for q in queries)
 
-    sql = (select([
-        text('most_filtering.quals'),
-        text('most_filtering.query'),
-        text('to_json(most_filtering) as "most filtering"'),
-        text('to_json(least_filtering) as "least filtering"'),
-        text('to_json(most_executed) as "most executed"'),
-        text('to_json(most_used) as "most used"')])
-           .select_from(
-               qual_constants(srvid, "most_filtering", tsfrom, tsto, condition,
-                              queries_str, quals)
-               .alias("most_filtering")
-               .join(
-                   qual_constants(srvid, "least_filtering",
-                                  tsfrom, tsto, condition,
-                                  queries_str, quals)
-                   .alias("least_filtering"),
-                   text("most_filtering.rownumber = "
-                        "least_filtering.rownumber"))
-               .join(qual_constants(srvid, "most_executed",
-                                    tsfrom, tsto, condition,
-                                    queries_str, quals)
-                     .alias("most_executed"),
-                     text("most_executed.rownumber = "
-                          "least_filtering.rownumber"))
-               .join(qual_constants(srvid, "most_used",
-                                    tsfrom, tsto, condition,
-                                    queries_str, quals)
-                     .alias("most_used"),
-                     text("most_used.rownumber = "
-                          "least_filtering.rownumber"))))
+    cols = [
+        "most_filtering.quals",
+        "most_filtering.query",
+        'to_json(most_filtering) as "most filtering"',
+        'to_json(least_filtering) as "least filtering"',
+        'to_json(most_executed) as "most executed"',
+        'to_json(most_used) as "most used"',
+    ]
 
-    params = {"server": srvid,
-              "database": database,
-              "from": tsfrom,
-              "to": tsto,
-              "queryids": queries}
-    quals = conn.execute(sql, params=params)
+    sql = """SELECT {cols}
+    FROM ({most_filtering}) AS most_filtering
+    JOIN ({least_filtering}) AS least_filtering USING (rownumber)
+    JOIN ({most_executed}) AS most_executed USING (rownumber)
+    JOIN ({most_used}) AS most_used USING (rownumber)""".format(
+        cols=", ".join(cols),
+        most_filtering=qual_constants(
+            srvid, "least_filtering", condition, queries_str, quals
+        ),
+        least_filtering=qual_constants(
+            srvid, "least_filtering", condition, queries_str, quals
+        ),
+        most_executed=qual_constants(
+            srvid, "most_executed", condition, queries_str, quals
+        ),
+        most_used=qual_constants(
+            srvid, "most_used", condition, queries_str, quals
+        ),
+    )
 
-    if quals.rowcount == 0:
+    params = {
+        "server": srvid,
+        "database": database,
+        "from": tsfrom,
+        "to": tsto,
+        "queryids": queries,
+    }
+    quals = cls.execute(sql, params=params)
+
+    if len(quals) == 0:
         return None
 
-    row = quals.first()
+    row = quals[0]
 
     return row
 
 
 class HypoPlan(JSONizable):
-
-    def __init__(self, baseplan, basecost,
-                 hypoplan, hypocost,
-                 query, indexes=None):
+    def __init__(
+        self, baseplan, basecost, hypoplan, hypocost, query, indexes=None
+    ):
         self.baseplan = baseplan
         self.basecost = basecost
         self.hypoplan = hypoplan
@@ -509,18 +587,18 @@ class HypoPlan(JSONizable):
 
     @property
     def gain_percent(self):
-        return round(100 - float(self.hypocost) * 100 / float(self.basecost), 2)
+        return round(
+            100 - float(self.hypocost) * 100 / float(self.basecost), 2
+        )
 
     def to_json(self):
         base = super(HypoPlan, self).to_json()
-        base['gain_percent'] = self.gain_percent
+        base["gain_percent"] = self.gain_percent
         return base
 
 
 class HypoIndex(JSONizable):
-
-    def __init__(self, nspname, relname, amname,
-                 composed_qual=None):
+    def __init__(self, nspname, relname, amname, composed_qual=None):
         self.nspname = nspname
         self.relname = relname
         self.qual = composed_qual
@@ -531,22 +609,23 @@ class HypoIndex(JSONizable):
         self._update_ddl()
 
     def _update_ddl(self):
-        if 'btree' == self.amname:
+        if "btree" == self.amname:
             attrs = []
             for qual in self.qual:
                 if qual.attname not in attrs:
                     attrs.append(qual.attname)
+            # Qual resolution is responsible for quoting all identifiers
             super(HypoIndex, self).__setattr__(
-                '_ddl',
-                """CREATE INDEX ON %s.%s(%s)""" % (
-                    quote_ident(self.nspname),
-                    quote_ident(self.relname),
-                    ",".join(attrs)))
+                "_ddl",
+                """CREATE INDEX ON {nsp}.{rel}({attrs})""".format(
+                    nsp=self.nspname, rel=self.relname, attrs=",".join(attrs)
+                ),
+            )
 
     def __setattr(self, name, value):
         super(HypoIndex, self).__setattr__(name, value)
         # Only btree is supported right now
-        if name in ('amname', 'nspname', 'relname', 'composed_qual'):
+        if name in ("amname", "nspname", "relname", "composed_qual"):
             self._update_ddl()
 
     @property
@@ -557,12 +636,16 @@ class HypoIndex(JSONizable):
     def hypo_ddl(self):
         ddl = self.ddl
         if ddl is not None:
-            return select([column("indexname")]
-                          ).select_from(func.hypopg_create_index(self.ddl))
+            return (
+                "SELECT indexname"
+                + " FROM {hypopg}.hypopg_create_index(%(sql)s)",
+                {"sql": ddl},
+            )
+        return (None, None)
 
     def to_json(self):
         base = super(HypoIndex, self).to_json()
-        base['ddl'] = self.ddl
+        base["ddl"] = self.ddl
         return base
 
 
@@ -576,20 +659,18 @@ def possible_indexes(composed_qual, order=()):
         else:
             # - attnum to guarantee stable sort
             return sys.maxsize - attnum
+
     for qual in sorted(composed_qual, key=sorter):
         for am in qual.amops.keys():
             by_am[am].append(qual)
     indexes = []
     for am, quals in by_am.items():
         base = quals[0]
-        indexes.append(HypoIndex(base.nspname,
-                                 base.relname,
-                                 am,
-                                 quals))
+        indexes.append(HypoIndex(base.nspname, base.relname, am, quals))
     return indexes
 
 
-def get_hypoplans(conn, query, indexes=None):
+def get_hypoplans(cur, query, indexes=None):
     """
     With a connection to a database where hypothetical indexes
     have already been created, request two plans for each query:
@@ -608,18 +689,41 @@ def get_hypoplans(conn, query, indexes=None):
     indexes = indexes or []
     # Escape literal '%'
     query = query.replace("%", "%%")
-    with conn.begin() as trans:
-        trans.execute("SET hypopg.enabled = off")
-        baseplan = "\n".join(v[0] for v in trans.execute("EXPLAIN %s" % query))
-        trans.execute("SET hypopg.enabled = on")
-        hypoplan = "\n".join(v[0] for v in trans.execute("EXPLAIN %s" % query))
+    cur.execute("SET hypopg.enabled = off")
+    try:
+        cur.execute("SAVEPOINT hypo")
+        cur.execute("EXPLAIN {}".format(query))
+        baseplan = "\n".join(v[0] for v in cur.fetchall())
+        cur.execute("RELEASE hypo")
+    except Exception as e:
+        cur.execute("ROLLBACK TO hypo")
+        raise e
+
+    cur.execute("SET hypopg.enabled = on")
+    try:
+        cur.execute("SAVEPOINT hypo")
+        cur.execute("EXPLAIN {}".format(query))
+        hypoplan = "\n".join(v[0] for v in cur.fetchall())
+        cur.execute("RELEASE hypo")
+    except Exception as e:
+        cur.execute("ROLLBACK TO hypo")
+        raise e
+
     COST_RE = "(?<=\.\.)\d+\.\d+"
     m = re.search(COST_RE, baseplan)
-    basecost = float(m.group(0))
+    if m:
+        basecost = float(m.group(0))
+    else:
+        basecost = None
     m = re.search(COST_RE, hypoplan)
-    hypocost = float(m.group(0))
+    if m:
+        hypocost = float(m.group(0))
+    else:
+        hypocost = None
     used_indexes = []
     for ind in indexes:
         if ind.name is not None and ind.name in hypoplan:
             used_indexes.append(ind)
-    return HypoPlan(baseplan, basecost, hypoplan, hypocost, query, used_indexes)
+    return HypoPlan(
+        baseplan, basecost, hypoplan, hypocost, query, used_indexes
+    )
