@@ -509,43 +509,49 @@ BASE_QUERY_WAIT_SAMPLE = """(
 # the highest transaction id is 2^32.
 def BASE_QUERY_PGSA_SAMPLE(per_db=False):
     if per_db:
-        extra = """JOIN {powa}.powa_catalog_databases d
-            ON d.oid = pgsa_history.datid
+        extra = """JOIN {powa}.powa_databases d
+            ON d.oid = pgsa_history.datid and d.srvid = pgsa_history.srvid
         WHERE d.datname = %(database)s"""
     else:
         extra = ""
 
     # We use dense_rank() as we need ALL the records for a specific ts
     return """
-    (SELECT pgsa_history.srvid,
-      dense_rank() OVER (ORDER BY pgsa_history.ts) AS number,
-      count(*) OVER () AS total,
-      ts,
-      datid,
-      cur_txid,
-      backend_xid,
-      backend_xmin,
-      backend_start,
-      xact_start,
-      query_start,
-      state,
-      leader_pid
+    (
+      SELECT *, max(number) OVER () AS total
       FROM (
-        SELECT *
+        SELECT pgsa_history.srvid,
+          dense_rank() OVER (ORDER BY pgsa_history.ts) AS number,
+          ts,
+          clock_ts,
+          datid,
+          cur_txid,
+          backend_xid,
+          backend_xmin,
+          backend_start,
+          xact_start,
+          query_start,
+          state,
+          leader_pid
         FROM (
-          SELECT srvid, (unnest(records)).*
-          FROM {{powa}}.powa_stat_activity_history pgsah
-          WHERE coalesce_range && tstzrange(%(from)s, %(to)s, '[]')
-          AND pgsah.srvid = %(server)s
-        ) AS unnested
-        WHERE ts <@ tstzrange(%(from)s, %(to)s, '[]')
-        UNION ALL
-        SELECT srvid, (record).*
-        FROM {{powa}}.powa_stat_activity_history_current pgsac
-        WHERE (pgsac.record).ts <@ tstzrange(%(from)s, %(to)s, '[]')
-        AND pgsac.srvid = %(server)s
-      ) AS pgsa_history
-      {extra}
+          SELECT *
+          FROM (
+            SELECT srvid, (unnest(records)).*
+            FROM {{powa}}.powa_stat_activity_history pgsah
+            WHERE coalesce_range && tstzrange(%(from)s, %(to)s, '[]')
+            AND pgsah.srvid = %(server)s
+          ) AS unnested
+          WHERE ts <@ tstzrange(%(from)s, %(to)s, '[]')
+          AND backend_type <> 'autovacuum worker'
+          UNION ALL
+          SELECT srvid, (record).*
+          FROM {{powa}}.powa_stat_activity_history_current pgsac
+          WHERE (pgsac.record).ts <@ tstzrange(%(from)s, %(to)s, '[]')
+          AND pgsac.srvid = %(server)s
+          AND (record).backend_type <> 'autovacuum worker'
+          ) AS pgsa_history
+          {extra}
+      ) AS ranking
     ) AS pgsa
     WHERE number %% ( int8larger((total)/(%(samples)s+1),1) ) = 0
 """.format(extra=extra)
@@ -645,8 +651,8 @@ BASE_QUERY_CHECKPOINTER_SAMPLE = """
 
 def BASE_QUERY_DATABASE_SAMPLE(per_db=False):
     if per_db:
-        extra = """JOIN {powa}.powa_catalog_databases d
-            ON d.oid = psd_history.datid
+        extra = """JOIN {powa}.powa_databases d
+            ON d.oid = psd_history.datid and d.srvid = psd_history.srvid
         WHERE d.datname = %(database)s"""
     else:
         extra = ""
@@ -706,8 +712,8 @@ def BASE_QUERY_DATABASE_SAMPLE(per_db=False):
 
 def BASE_QUERY_DATABASE_CONFLICTS_SAMPLE(per_db=False):
     if per_db:
-        extra = """JOIN {powa}.powa_catalog_databases d
-            ON d.oid = psd_history.datid
+        extra = """JOIN {powa}.powa_databases d
+            ON d.oid = psd_history.datid and d.srvid = psd_history.srvid
         WHERE d.datname = %(database)s"""
     else:
         extra = ""
@@ -1326,7 +1332,7 @@ def powa_get_pgsa_sample(per_db=False):
 
     def ts_get_sec(field):
         alias = field + "_age"
-        return """extract(epoch FROM (ts - {f})) * 1000 AS {a}""".format(
+        return """extract(epoch FROM (coalesce(clock_ts, ts) - {f})) * 1000 AS {a}""".format(
             f=field, a=alias
         )
 
