@@ -6,9 +6,12 @@ from tornado.options import options
 
 if __name__ == "__main__":
     application = make_app(debug=True, gzip=True, compress_response=True)
-    application.listen(8888)
+    application.listen(options.port, address=options.address)
     logger = logging.getLogger("tornado.application")
     logger.info(
-        "Starting powa-web on http://127.0.0.1:8888%s", options.url_prefix
+        "Starting powa-web on http://%s:%s%s",
+        options.address,
+        options.port,
+        options.url_prefix,
     )
     tornado.ioloop.IOLoop.instance().start()
